@@ -8,9 +8,16 @@
 #include <ctype.h>
 #include <math.h>
 #include <libgnomeprint/gnome-font.h>
-#include <gnome-xml/tree.h>
+
+#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
+#include <libxml/parser.h>
+#include <libxml/parser.h>
+#define root children
+#define childs children
+#else
 #include <gnome-xml/parser.h>
 #include <gnome-xml/xmlmemory.h>
+#endif
 
 int _gnome_print_debug = -1;
 
@@ -516,6 +523,9 @@ gnome_font_load_fontmap (GnomeFontClass *class, const char *fn)
   debugmsg ("filename %s\n", fn);
 #endif
 
+#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
+  xmlKeepBlanksDefault(0);
+#endif
   doc = xmlParseFile( fn );
   if ( doc && doc->root && doc->root->name && ( ! strcmp( doc->root->name, "fontmap" ) ) ) /* && doc->root->ns && doc->root->ns->href && ( ! strcmp( doc->root->ns->href, "http://www.gnome.org/gnome-font/0.0" ) ) ) */
     {
