@@ -47,23 +47,38 @@ typedef struct _GnomePrintDialoguePrivate {
 guint
 gnome_print_dialog_get_type (void)
 {
-  static guint select_paper_type = 0;
+	static guint select_paper_type = 0;
   
-  if (!select_paper_type) {
-    GtkTypeInfo select_paper_info = {
-      "GnomePrintDialog",
-      sizeof (GnomePrintDialog),
-      sizeof (GnomePrintDialogClass),
-      (GtkClassInitFunc) gnome_print_dialog_class_init,
-      (GtkObjectInitFunc) gnome_print_dialog_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
-    };
-    
-    select_paper_type = gtk_type_unique (gnome_dialog_get_type (), &select_paper_info);
-  }
+	if (!select_paper_type) {
+		GtkTypeInfo select_paper_info = {
+			"GnomePrintDialog",
+			sizeof (GnomePrintDialog),
+			sizeof (GnomePrintDialogClass),
+			(GtkClassInitFunc) gnome_print_dialog_class_init,
+			(GtkObjectInitFunc) gnome_print_dialog_init,
+			(GtkArgSetFunc) NULL,
+			(GtkArgGetFunc) NULL
+		};
+		select_paper_type = gtk_type_unique (gnome_dialog_get_type (), &select_paper_info);
+	}
   
-  return select_paper_type;
+	return select_paper_type;
+}
+
+static void
+gnome_print_dialog_destroy (GtkObject *object)
+{
+	GnomePrintDialog *gpd;
+	
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_DIALOG (object));
+	
+	gpd = GNOME_PRINT_DIALOG (object);
+	if (gpd->range_accel_group)
+		gtk_accel_group_unref (gpd->range_accel_group);
+  
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 static void
@@ -74,7 +89,6 @@ gnome_print_dialog_finalize (GtkObject *object)
 	g_free(gpd->priv);
 
 	/* all widgets should be free'd by their containers? */
-
 	GTK_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -87,6 +101,7 @@ gnome_print_dialog_class_init (GnomePrintDialogClass *class)
 	object_class = (GtkObjectClass *) class;
 	parent_class = gtk_type_class (gnome_dialog_get_type ());
 
+	object_class->destroy = gnome_print_dialog_destroy;
 	object_class->finalize = gnome_print_dialog_finalize;
 }
 
@@ -98,13 +113,15 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 	gtk_widget_show(gpd->printer);
 	gpd->range_type = GNOME_PRINT_RANGETYPE_NONE;
 	gpd->priv = g_malloc0(sizeof(Private));
+	gpd->range_accel_group = NULL;
 }
 
 #define PAD 4
 static void init_body(GnomePrintDialog *gpd, int flags)
 {
 	GtkWidget *hbox;
-
+	GnomePrinterWidget *gpw = GNOME_PRINTER_WIDGET (gpd->printer);
+	
 	if (gpd->copies || gpd->range_container)
 		return;
 
@@ -113,17 +130,36 @@ static void init_body(GnomePrintDialog *gpd, int flags)
 		gtk_box_pack_start(GTK_BOX(GNOME_DIALOG (gpd)->vbox), GTK_WIDGET (hbox), FALSE, FALSE, 3);
 
 		if (flags & GNOME_PRINT_DIALOG_COPIES) {
+			GtkSpinButton *sb;
+			GnomePrintCopies *gpc;
+			
 			gpd->copies = gnome_print_copies_new ();
 			gtk_box_pack_end(GTK_BOX(hbox), gpd->copies, FALSE, TRUE, 3);
+			gpc = GNOME_PRINT_COPIES (gpd->copies);
+			sb = GTK_SPIN_BUTTON (gpc->copies);
+			gnome_dialog_editable_enters
+				(GNOME_DIALOG(gpd), GTK_EDITABLE(&sb->entry));
+			gtk_window_add_accel_group (GTK_WINDOW (gpd),
+						    gpc->accel_group);
 		}
 		
 		if (flags & GNOME_PRINT_DIALOG_RANGE) {
 			gpd->range_container = gtk_frame_new(_("Print Range"));
 			gtk_box_pack_start(GTK_BOX(hbox), gpd->range_container, FALSE, TRUE, 3);
 			gtk_widget_show(gpd->range_container);
+			gpd->range_accel_group = gtk_accel_group_new ();
+			gtk_window_add_accel_group (GTK_WINDOW (gpd),
+						    gpd->range_accel_group);
 		}
 		gtk_widget_show_all(hbox);
 	}
+	gnome_dialog_editable_enters (GNOME_DIALOG (gpd),
+				      GTK_EDITABLE (gpw->entry_command));
+	gnome_dialog_editable_enters (GNOME_DIALOG (gpd),
+				      GTK_EDITABLE (gpw->entry_filename));
+	
+	gtk_window_add_accel_group (GTK_WINDOW (gpd), gpw->accel_group);
+	gtk_widget_grab_focus (gpw->profile_selector);
 }
 
 static const char *print_buttons[] = { _("Print"), _("Preview"), GNOME_STOCK_BUTTON_CANCEL, NULL };
@@ -228,6 +264,7 @@ gnome_print_dialog_construct_range_any(GnomePrintDialog *gpd, int flags, GtkWidg
 	GtkWidget *table;
 	GtkWidget *current = NULL, *all = NULL, *range = NULL, *selection = NULL;
 	int row;
+	guint accel_key;
 	Private *p;
 	GSList *group= NULL;
 
@@ -250,31 +287,60 @@ gnome_print_dialog_construct_range_any(GnomePrintDialog *gpd, int flags, GtkWidg
 	table = gtk_table_new(row, 2, FALSE);
 	row=0;
 	if (flags&GNOME_PRINT_RANGE_CURRENT) {
-		current = gtk_radio_button_new_with_label(group, currentlabel);
+		current = gtk_radio_button_new_with_label(group, "");
+		accel_key = gtk_label_parse_uline
+			(GTK_LABEL (GTK_BIN (current)->child), currentlabel);
 		gtk_table_attach(GTK_TABLE(table), current, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
 		group = gtk_radio_button_group((GtkRadioButton *)current);
+		if (accel_key != GDK_VoidSymbol)
+			gtk_widget_add_accelerator (current, "clicked",
+						    gpd->range_accel_group,
+						    accel_key,
+						    GDK_MOD1_MASK, 0);
 		row++;
 	}
 
 	if (flags&GNOME_PRINT_RANGE_ALL) {
-		all = gtk_radio_button_new_with_label(group, _("All"));
+		all = gtk_radio_button_new_with_label(group, "");
+		accel_key = gtk_label_parse_uline
+			(GTK_LABEL (GTK_BIN (all)->child), _("_All"));
 		gtk_table_attach(GTK_TABLE(table), all, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
 		group = gtk_radio_button_group((GtkRadioButton *)all);
+		if (accel_key != GDK_VoidSymbol)
+			gtk_widget_add_accelerator (all, "clicked",
+						    gpd->range_accel_group,
+						    accel_key,
+						    GDK_MOD1_MASK, 0);
 		row++;
 	}
 
 	if (flags&GNOME_PRINT_RANGE_RANGE) {
-		range = gtk_radio_button_new_with_label(group, rangelabel);
+		range = gtk_radio_button_new_with_label(group, "");
+		accel_key = gtk_label_parse_uline
+			(GTK_LABEL (GTK_BIN (range)->child), rangelabel);
 		gtk_table_attach(GTK_TABLE(table), range, 0, 1, row, row+1, GTK_FILL, GTK_SHRINK, 0, 0);
 		gtk_table_attach(GTK_TABLE(table), range_widget, 1, 2, row, row+1, GTK_FILL, 0, 0, 0);
 		group = gtk_radio_button_group((GtkRadioButton *)range);
+		if (accel_key != GDK_VoidSymbol)
+			gtk_widget_add_accelerator (range, "clicked",
+						    gpd->range_accel_group,
+						    accel_key,
+						    GDK_MOD1_MASK, 0);
 		row++;
 	}
 
 	if (flags&GNOME_PRINT_RANGE_SELECTION) {
-		selection = gtk_radio_button_new_with_label(group, _("Selection"));
+		selection = gtk_radio_button_new_with_label(group, "");
+		accel_key = gtk_label_parse_uline
+			(GTK_LABEL (GTK_BIN (selection)->child),
+			 _("_Selection"));
 		gtk_table_attach(GTK_TABLE(table), selection, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
 		group = gtk_radio_button_group((GtkRadioButton *)selection);
+		if (accel_key != GDK_VoidSymbol)
+			gtk_widget_add_accelerator (selection, "clicked",
+						    gpd->range_accel_group,
+						    accel_key,
+						    GDK_MOD1_MASK, 0);
 		row++;
 	}
 
@@ -375,6 +441,8 @@ gnome_print_dialog_construct_range_page(GnomePrintDialog *gpd, int flags, int st
 	GtkWidget *hbox = NULL, *label;
 	GtkWidget *range_start = NULL, *range_end = NULL;
 	GtkAdjustment *adj_start, *adj_end;
+	GtkEntry *entry;
+	guint label_key;
 	Private *p;
 
 	g_return_if_fail(gpd != NULL);
@@ -385,19 +453,39 @@ gnome_print_dialog_construct_range_page(GnomePrintDialog *gpd, int flags, int st
 
 	if (flags&GNOME_PRINT_RANGE_RANGE) {
 		hbox = gtk_hbox_new(FALSE, 3);
-		label = gtk_label_new(_("from:"));
+		label = gtk_label_new("");
+		label_key = gtk_label_parse_uline (GTK_LABEL (label),
+						   _("f_rom:"));
 		gtk_box_pack_start((GtkBox *)hbox, label, FALSE, FALSE, 0);
 
 		adj_start = (GtkAdjustment *)gtk_adjustment_new(start, start, end, 1, 10, 10);
 		range_start = gtk_spin_button_new(adj_start, 1, 0.0);
 		gtk_box_pack_start((GtkBox *)hbox, range_start, FALSE, FALSE, 0);
-
-		label = gtk_label_new(_("to:"));
+		if (label_key != GDK_VoidSymbol) {
+			entry = &GTK_SPIN_BUTTON (range_start)->entry;
+			gtk_widget_add_accelerator (GTK_WIDGET (entry),
+						    "grab_focus",
+						    gpd->range_accel_group,
+						    label_key, GDK_MOD1_MASK,
+						    0);
+		}
+	
+		label = gtk_label_new("");
+		label_key = gtk_label_parse_uline (GTK_LABEL (label),
+						   _("_to:"));
 		gtk_box_pack_start((GtkBox *)hbox, label, FALSE, FALSE, 0);
 
 		adj_end = (GtkAdjustment *)gtk_adjustment_new(end, start, end, 1, 10, 10);
 		range_end = gtk_spin_button_new(adj_end, 1, 0.0);
 		gtk_box_pack_start((GtkBox *)hbox, range_end, FALSE, FALSE, 0);
+		if (label_key != GDK_VoidSymbol) {
+			entry = &GTK_SPIN_BUTTON (range_end)->entry;
+			gtk_widget_add_accelerator (GTK_WIDGET (entry),
+						    "grab_focus",
+						    gpd->range_accel_group,
+						    label_key, GDK_MOD1_MASK,
+						    0);
+		}
 
 	}
 
@@ -433,14 +521,18 @@ gnome_print_dialog_get_range(GnomePrintDialog *gpd)
 
 	p = gpd->priv;
 
-	if (GTK_TOGGLE_BUTTON(p->current)->active)
-		return GNOME_PRINT_RANGE_CURRENT;
-	if (GTK_TOGGLE_BUTTON(p->all)->active)
-		return GNOME_PRINT_RANGE_ALL;
-	if (GTK_TOGGLE_BUTTON(p->range)->active)
-		return GNOME_PRINT_RANGE_RANGE;
-	if (GTK_TOGGLE_BUTTON(p->selection)->active)
-		return GNOME_PRINT_RANGE_SELECTION;
+	if (p->current!=NULL)
+	     if (GTK_TOGGLE_BUTTON(p->current)->active)
+		  return GNOME_PRINT_RANGE_CURRENT;
+	if (p->all!=NULL)
+	     if (GTK_TOGGLE_BUTTON(p->all)->active)
+		  return GNOME_PRINT_RANGE_ALL;
+	if (p->range!=NULL)
+	     if (GTK_TOGGLE_BUTTON(p->range)->active)
+		  return GNOME_PRINT_RANGE_RANGE;
+	if (p->selection!=NULL)
+	     if (GTK_TOGGLE_BUTTON(p->selection)->active)
+		  return GNOME_PRINT_RANGE_SELECTION;
 	return 0;
 }
 
