@@ -70,6 +70,9 @@ gnome_print_master_finalize (GtkObject *object)
 	if (gpm->context != NULL)
 		gtk_object_unref (GTK_OBJECT (gpm->context));
 
+	if (gpm->printer)
+		gtk_object_unref (GTK_OBJECT (gpm->printer));
+
 	GTK_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -196,12 +199,17 @@ gnome_print_master_set_paper(GnomePrintMaster *gpm, const GnomePaper *paper)
  * 
  * Sets physical printer used to print to.  If this is not set,
  * then a dialogue will be presented on each print.
+ *
+ * This functions takes ownership of the printer reference.
  **/
 void
 gnome_print_master_set_printer(GnomePrintMaster *gpm, GnomePrinter *printer)
 {
 	g_return_if_fail (GNOME_IS_PRINT_MASTER (gpm));
 
+	if (gpm->printer)
+		gtk_object_unref (GTK_OBJECT (gpm->printer));
+
 	gpm->printer = printer;
 }
 
@@ -286,9 +294,11 @@ gnome_print_master_print(GnomePrintMaster *gpm)
 		if (printer == NULL) {
 			return -1;
 		}
-	}
+	} else
+		gtk_object_ref (GTK_OBJECT (printer));
+
 	output = gnome_print_context_new_with_paper_size(printer, gnome_paper_name (gpm->paper));
-	
+
 	if (gpm->iscollate) {
 		repeat = 1;
 		loop = gpm->copies;
@@ -305,5 +315,8 @@ gnome_print_master_print(GnomePrintMaster *gpm)
 			}
 		}
 	}
+
+	gtk_object_unref (GTK_OBJECT (printer));
+
 	return gnome_print_context_close(output);
 }
