@@ -2,8 +2,9 @@
  * Metafile printer driver.
  *
  * Authors:
- *    Miguel de Icaza (miguel@gnu.org
- *    michael Zucchi <notzed@helixcode.com>
+ *    Miguel de Icaza (miguel@gnu.org)
+ *    Michael Zucchi <notzed@helixcode.com>
+ *    Morten Welinder (terra@diku.dk)
  *
  * TODO:
  *    If the GnomeFont is not found during the replay of a metafile
@@ -18,7 +19,7 @@
  *    Because each page is played back in isolation, if there are any
  *    options that persist across showpage's, then they need to be
  *    tracked, and stored in the page header for each page.  But I
- *    dont think there are ...
+ *    dont think there are...
  */
 #include <config.h>
 #include <gtk/gtk.h>
@@ -85,10 +86,12 @@ typedef enum {
 	GNOME_META_EOCLIP,
 	GNOME_META_GRAYIMAGE,
 	GNOME_META_RGBIMAGE,
-	GNOME_META_SHOWPAGE,
-	GNOME_META_BEGINPAGE,
 	GNOME_META_TEXTLINE,
+	GNOME_META_BEGINPAGE,
+	GNOME_META_SHOWPAGE,
 	GNOME_META_CLOSE,
+	GNOME_META_SETOPACITY,
+	GNOME_META_RGBAIMAGE
 } GnomeMetaType;
 
 #ifdef META_DEBUG
@@ -118,13 +121,21 @@ static char *meta_type_names[] = {
 	"EOCLIP",
 	"GRAYIMAGE",
 	"RGBIMAGE",
-	"SHOWPAGE",
-	"BEGINPAGE",
 	"TEXTLINE",
-	"CLOSE"
+	"BEGINPAGE",
+	"SHOWPAGE",
+	"CLOSE",
+	"SETOPACITY",
+	"RGBAIMAGE"
 };
 #endif
-	
+
+typedef enum {
+	GNOME_META_DOUBLE_INT,        /* An integer.  */
+	GNOME_META_DOUBLE_INT1000,    /* An integer to be divided by 1000.  */
+	GNOME_META_DOUBLE_I386        /* IEEE-xxxx little endian.  */
+} GnomeMetaDoubleType;
+
 static void
 gnome_print_meta_finalize (GtkObject *object)
 {
@@ -149,7 +160,7 @@ check_room (GnomePrintMeta *meta, int size)
 	if (bytes_left < size){
 		int min_size = MAX (BLOCKSIZE, size-bytes_left);
 		void *newp;
-		
+
 		newp = g_realloc (meta->buffer, meta->buffer_size + min_size);
 		if (!newp)
 			return FALSE;
@@ -163,7 +174,7 @@ check_room (GnomePrintMeta *meta, int size)
 static void
 encode_block (GnomePrintMeta *meta, int size, void const *data)
 {
-	if (!check_room(meta, size))
+	if (!check_room (meta, size))
 		return;
 
 	memcpy (meta->buffer + meta->current, data, size);
@@ -174,56 +185,81 @@ static void
 encode_int_header (GnomePrintMeta *pc, gint32 value)
 {
 	gint32 new = g_htonl (value);
-	
+
 	encode_block (GNOME_PRINT_META (pc), sizeof (gint32), &new);
 }
 
 static void
 encode_int (GnomePrintContext *pc, gint32 value)
 {
-	char *out;
-	GnomePrintMeta *meta = GNOME_PRINT_META(pc);
+	char *out, *out0;
+	guint32 vabs, mask;
+	int bits;
 
-	if (!check_room(meta, 6))
+	GnomePrintMeta *meta = GNOME_PRINT_META (pc);
+	if (!check_room (meta, sizeof (value) * 8 / 7 + 1))
 		return;
 
-	/* we encode upto 7 bits of info in each byte, using the
-	   top bit to indicate the last byte, this makes decoding easier */
-	out = meta->buffer + meta->current;
-	if (value< (1<<7)) {
-		*out++ = value | 0x80;
-	} else if (value < (1<<14)) {
-		*out++ = value >> 7;
-		*out++ = (value & 0x7f) | 0x80;
-	} else if (value < (1<<21)) {
-		*out++ = value >> 14;
-		*out++ = (value >> 7) & 0x7f;
-		*out++ = (value & 0x7f) | 0x80;
-	} else if (value < (1<<28)) {
-		*out++ = value >> 21;
-		*out++ = (value >> 14) & 0x7f;
-		*out++ = (value >> 7) & 0x7f;
-		*out++ = (value & 0x7f) | 0x80;
-	} else {
-		*out++ = (value >> 28) & 0x7f; /* c doesn't say what happens to the sign bit, even for unsigned */
-		*out++ = (value >> 21) & 0x7f;
-		*out++ = (value >> 14) & 0x7f;
-		*out++ = (value >> 7) & 0x7f;
-		*out++ = (value & 0x7f) | 0x80;
-	}
+	/*
+	 * We encode an integer as a sequence of bytes where all but the
+	 * last have the high bit set to zero.  The final byte does have
+	 * that bit set.
+	 *
+	 * Bit 6 of the first byte contains the sign bit.
+	 *
+	 * The remaining 6, 7, ..., 7 bits contain the absolute value,
+	 * starting with the six least significant bits in the first
+	 * byte.
+	 */
+	out0 = out = meta->buffer + meta->current;
+	vabs = (value >= 0) ? value : -value;
+	bits = 6;
+	mask = 0x3f;
+
+	do {
+		*out++ = (vabs & mask);
+		vabs >>= bits;
+		bits = 7;
+		mask = 0x7f;
+	} while (vabs);
+
+	out[-1] |= 0x80;
+	if (value < 0) out0[0] |= 0x40;
 	meta->current = out - meta->buffer;
 }
 
 static void
 encode_double (GnomePrintContext *pc, double d)
 {
-	static int message_shown;
+	int sd = sizeof (d);
 
-	if (!message_shown){
-		g_warning ("We lack marshalling of type `double'\n");
-		message_shown = 1;
+	if (d == (gint32)d) {
+		encode_int (pc, GNOME_META_DOUBLE_INT);
+		encode_int (pc, (int)d);
+	} else {
+		double d1000 = d * 1000;
+		if (d1000 == (gint32)d1000) {
+			encode_int (pc, GNOME_META_DOUBLE_INT1000);
+			encode_int (pc, (int)d1000);
+		} else {
+			encode_int (pc, GNOME_META_DOUBLE_I386);
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+			{
+				int     i;
+				guint8 *t  = (guint8 *)&d;
+				guint8 block[sizeof (d)];
+
+				for (i = 0; i < sd; i++)
+					block[sd - 1 - i] = t[i];
+				encode_block (GNOME_PRINT_META (pc), sd, &block);
+			}
+#elif G_BYTE_ORDER == G_LITTLE_ENDIAN
+			encode_block (GNOME_PRINT_META (pc), sd, &d);
+#else
+#error encode_double_needs_attention
+#endif
+		}
 	}
-	encode_block (GNOME_PRINT_META (pc), sizeof (double), &d); 
 }
 
 static void
@@ -238,19 +274,28 @@ encode_six_doubles (GnomePrintContext *pc, double x1, double y1, double x2, doub
 }
 
 static void
-encode_header(GnomePrintMeta *pc, GnomeMetaFileHeader *mh)
+encode_string (GnomePrintContext *pc, const char *str)
 {
-	encode_block(pc, sizeof(mh->signature), mh->signature);
-	encode_int_header(pc, mh->size);
+	int len;
+	len = strlen (str);
+	encode_int (pc, len);
+	encode_block (GNOME_PRINT_META (pc), len, str);
+}
+
+static void
+encode_header (GnomePrintMeta *pc, GnomeMetaFileHeader *mh)
+{
+	encode_block (pc, sizeof (mh->signature), mh->signature);
+	encode_int_header (pc, mh->size);
 }
 
 /* sets the size of the last page */
 static void
-close_page_header(GnomePrintMeta *pc)
+close_page_header (GnomePrintMeta *pc)
 {
 	if (pc->priv->last_page != -1) {
 		gint32 l;
-		l = g_htonl(pc->current - pc->priv->last_page - PAGEHEADER_SIZE);
+		l = g_htonl (pc->current - pc->priv->last_page - PAGEHEADER_SIZE);
 		/* we cannot map the page header back into memory, so poke it back in */
 		/* Note: unaligned access -- MW.  */
 		memcpy ((pc->buffer + pc->priv->last_page) + 4, &l, sizeof (l));
@@ -258,12 +303,12 @@ close_page_header(GnomePrintMeta *pc)
 }
 
 static void
-encode_page_header(GnomePrintMeta *pc, GnomeMetaPageHeader *mh)
+encode_page_header (GnomePrintMeta *pc, GnomeMetaPageHeader *mh)
 {
-	close_page_header(pc);
+	close_page_header (pc);
 	pc->priv->last_page = pc->current;
-	encode_block(pc, sizeof(mh->signature), mh->signature);
-	encode_int_header(pc, mh->size);
+	encode_block (pc, sizeof (mh->signature), mh->signature);
+	encode_int_header (pc, mh->size);
 #ifdef META_DEBUG
 	g_warning ("Page header ends at 0x%x", pc->current);
 #endif
@@ -389,7 +434,7 @@ static int
 meta_setdash (GnomePrintContext *pc, int n_values, const double *values, double offset)
 {
 	int i;
-	
+
 	encode_int (pc, GNOME_META_SETDASH);
 	encode_int (pc, n_values);
 
@@ -417,17 +462,14 @@ static int
 meta_setfont (GnomePrintContext *pc, GnomeFont *font)
 {
 	const char *fontname = font->fontmap_entry->font_name;
-	int len = strlen (fontname);
-			  
+
 #ifdef META_DEBUG
 	g_warning ("SetFont starts at 0x%x",
 		   GNOME_PRINT_META (pc)->current);
 #endif
 	encode_int (pc, GNOME_META_SETFONT);
 	encode_double (pc, font->size);
-	encode_int (pc, len);
-	encode_block (GNOME_PRINT_META (pc), len, fontname);
-	
+	encode_string (pc, fontname);
 	return 0;
 }
 
@@ -435,15 +477,12 @@ meta_setfont (GnomePrintContext *pc, GnomeFont *font)
 static int
 meta_show (GnomePrintContext *pc, char const *text)
 {
-	int len = strlen (text);
-	
 #ifdef META_DEBUG
 	g_warning ("Show starts at 0x%x",
 		   GNOME_PRINT_META (pc)->current);
 #endif
 	encode_int (pc, GNOME_META_SHOW);
-	encode_int (pc, len);
-	encode_block (GNOME_PRINT_META (pc), len, text);
+	encode_string (pc, text);
 	return 0;
 }
 
@@ -495,7 +534,7 @@ static void
 encode_image (GnomePrintContext *pc, const char *data, int width, int height, int rowstride, int bytes_per_pixel)
 {
 	int ix, y;
-	
+
 	encode_int (pc, height);
 	encode_int (pc, width);
 
@@ -526,7 +565,7 @@ static int
 meta_textline (GnomePrintContext *pc, GnomeTextLine *line)
 {
 	encode_int (pc, GNOME_META_TEXTLINE);
-	g_warning ("meta_textline: not implemented yet\n");
+	g_warning ("meta_textline: not implemented yet");
 	return 0;
 }
 
@@ -537,7 +576,7 @@ meta_showpage (GnomePrintContext *pc)
 		PAGE_SIGNATURE,
 		-1
 	};
-	GnomePrintMeta *meta = GNOME_PRINT_META(pc);
+	GnomePrintMeta *meta = GNOME_PRINT_META (pc);
 
 #ifdef META_DEBUG
 	g_warning ("ShowPage starts at 0x%x",
@@ -546,7 +585,7 @@ meta_showpage (GnomePrintContext *pc)
 	encode_int (pc, GNOME_META_SHOWPAGE);
 
 	/* assume another page is coming */
-	encode_page_header(meta, &page_header);
+	encode_page_header (meta, &page_header);
 
 	meta->pages++;
 	return 0;
@@ -555,11 +594,12 @@ meta_showpage (GnomePrintContext *pc)
 static int
 meta_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
 {
-	int len = strlen (name_of_this_page);
-
+#ifdef META_DEBUG
+	g_warning ("BeginPage starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_BEGINPAGE);
-	encode_int (pc, len);
-	encode_block (GNOME_PRINT_META (pc), len, name_of_this_page);
+	encode_string (pc, name_of_this_page);
 	return 0;
 }
 
@@ -579,19 +619,36 @@ meta_close (GnomePrintContext *pc)
 	return 0;
 }
 
+static int
+meta_setopacity (GnomePrintContext *pc, double opacity)
+{
+	encode_int (pc, GNOME_META_SETOPACITY);
+	encode_double (pc, opacity);
+	return 0;
+}
+
+static int
+meta_rgbaimage (GnomePrintContext *pc, const char *data,
+		int width, int height, int rowstride)
+{
+	encode_int (pc, GNOME_META_RGBAIMAGE);
+	encode_image (pc, data, width, height, rowstride, 4);
+	return 0;
+}
+
 static void
 gnome_print_meta_class_init (GnomePrintMetaClass *class)
 {
 	GtkObjectClass *object_class;
 	GnomePrintContextClass *pc_class;
-	
+
 	object_class = (GtkObjectClass *)class;
 	pc_class = (GnomePrintContextClass *)class;
-	
+
 	parent_class = gtk_type_class (gnome_print_context_get_type ());
-	
+
 	object_class->finalize = gnome_print_meta_finalize;
-	
+
 	/* initialization code, autogenned */
 	pc_class->newpath = meta_newpath;
 	pc_class->moveto = meta_moveto;
@@ -619,10 +676,11 @@ gnome_print_meta_class_init (GnomePrintMetaClass *class)
 	pc_class->grayimage = meta_grayimage;
 	pc_class->rgbimage = meta_rgbimage;
 	pc_class->textline = meta_textline;
-	pc_class->showpage = meta_showpage;
 	pc_class->beginpage = meta_beginpage;
-	
+	pc_class->showpage = meta_showpage;
 	pc_class->close = meta_close;
+	pc_class->setopacity = meta_setopacity;
+	pc_class->rgbaimage = meta_rgbaimage;
 }
 
 static void
@@ -639,10 +697,10 @@ gnome_print_meta_init (GnomePrintMeta *meta)
 
 	meta->buffer_size = BLOCKSIZE;
 	meta->buffer = g_malloc (meta->buffer_size);
-	meta->priv = g_malloc0 (sizeof(GnomePrintMetaPrivate));
+	meta->priv = g_malloc0 (sizeof (GnomePrintMetaPrivate));
 	meta->priv->last_page = -1;
-	encode_header(meta, &header);
-	encode_page_header(meta, &page_header);
+	encode_header (meta, &header);
+	encode_page_header (meta, &page_header);
 }
 
 /**
@@ -654,7 +712,7 @@ GtkType
 gnome_print_meta_get_type (void)
 {
 	static GtkType meta_type = 0;
-	
+
 	if (!meta_type){
 		GtkTypeInfo meta_info =
 		{
@@ -667,10 +725,10 @@ gnome_print_meta_get_type (void)
 			/* reserved_2 */ NULL,
 			(GtkClassInitFunc) NULL,
 		};
-		
+
 		meta_type = gtk_type_unique (gnome_print_context_get_type (), &meta_info);
 	}
-	
+
 	return meta_type;
 }
 
@@ -707,7 +765,7 @@ gnome_print_meta_new_from (const void *data)
 
 	g_return_val_if_fail (data != NULL, NULL);
 
-	decode_header(data, &header);
+	decode_header (data, &header);
 	if (strncmp (header.signature, GNOME_METAFILE_SIGNATURE, GNOME_METAFILE_SIGNATURE_SIZE) != 0)
 		return NULL;
 
@@ -734,7 +792,7 @@ gnome_print_meta_new_from (const void *data)
  *          point to the new buffer
  * @buflen: pointer to an integer that will be set to the lenght of buffer.
  *
- * Makes buffer point to the internal GnomePrintMeta information. 
+ * Makes buffer point to the internal GnomePrintMeta information.
  *
  * Returns 0 on a failure, any other value on success
  */
@@ -747,7 +805,7 @@ gnome_print_meta_access_buffer (GnomePrintMeta *meta, void **buffer, int *buflen
 	g_return_val_if_fail (GNOME_IS_PRINT_META (meta), 0);
 
 	p = *buffer = meta->buffer;
-	*((gint *)(p + FILEHEADER_SIZE)) = g_htonl(meta->current);
+	*((gint *)(p + FILEHEADER_SIZE)) = g_htonl (meta->current);
 	*buflen = meta->current;
 
 	return 1;
@@ -777,7 +835,7 @@ gnome_print_meta_get_copy (GnomePrintMeta *meta, void **buffer, int *buflen)
 		return 0;
 
 	memcpy (p, meta->buffer, meta->buffer_size);
-	*((gint *)(p + FILEHEADER_SIZE)) = g_htonl(meta->current);
+	*((gint *)(p + FILEHEADER_SIZE)) = g_htonl (meta->current);
 	*buflen = meta->current;
 
 	return 1;
@@ -797,27 +855,67 @@ decode_int_header (const char *data, gint32 *dest)
 static const char *
 decode_int (const char *data, gint32 *dest)
 {
-	gint32 value=0, v;
+	guint32 vabs = 0, mask = 0x3f;
+	int bits = 6, shift = 0;
+	int neg;
+	char c;
 
-	/* until we get the last byte, keep decoding 7 bits at a time */
-	while ( ((v = *data++) & 0x80) == 0) {
-		value |= v;
-		value <<= 7;
-	}
-	value |= (v&0x7f);
-	*dest = value;
+	neg = (*data & 0x40);
+	do {
+		vabs |= ((c = *data++) & mask) << shift;
+		shift += bits;
+		bits = 7;
+		mask = 0x7f;
+	} while ((c & 0x80) == 0);
+
+	*dest = neg ? -vabs : vabs;
 	return data;
 }
 
 static const char *
 decode_double (const char *data, double *dest)
 {
-	/* Since the size is known, this should not be as bad as it
-	   looks.  Check the generated assembler before you come up
-	   with non-ANSI hacks.  -- MW.  */
-	/* Should we do something like g_ntohl?  */
-	memcpy (dest, data, sizeof (double));
-	return data + sizeof (double);
+	int i;
+	int sd = sizeof (double);
+	data = decode_int (data, &i);
+
+	switch ((GnomeMetaDoubleType)i) {
+	case GNOME_META_DOUBLE_INT:
+		data = decode_int (data, &i);
+		*dest = i;
+		break;
+	case GNOME_META_DOUBLE_INT1000:
+		data = decode_int (data, &i);
+		*dest = i / 1000.0;
+		break;
+	case GNOME_META_DOUBLE_I386:
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+		for (i = 0; i < sd; i++)
+			((guint8 *)dest)[sd - 1 - i] = data[i];
+		data += sd;
+#elif G_BYTE_ORDER == G_LITTLE_ENDIAN
+		memcpy (dest, data, sd);
+		data += sd;
+#else
+#error decode_double_needs_attention
+#endif
+		break;
+	default:
+		*dest = 0;   /* ??? */
+	}
+
+	return data;
+}
+
+static const char *
+decode_string (const char *data, char **dest)
+{
+	int len;
+	data = decode_int (data, &len);
+	*dest = g_malloc (len + 1);
+	memcpy (*dest, data, len);
+	(*dest)[len] = 0;
+	return data + len;
 }
 
 static const char *
@@ -826,19 +924,19 @@ decode_image (const char *data, GnomePrintContext *dest, int bytes_per_pixel)
 	void *buffer;
 	gint32 height, width;
 	int size;
-	
+
 	data = decode_int (data, &height);
 	data = decode_int (data, &width);
 
 	size = height * width * bytes_per_pixel;
 	buffer = g_malloc (size);
 	memcpy (buffer, data, size);
-	
+
 	if (bytes_per_pixel == 1)
 		gnome_print_grayimage (dest, buffer, width, height, width);
 	else
 		gnome_print_rgbimage (dest, buffer, width, height, width * bytes_per_pixel);
-	
+
 	g_free (buffer);
 	return data + size;
 }
@@ -846,36 +944,36 @@ decode_image (const char *data, GnomePrintContext *dest, int bytes_per_pixel)
 static const char *
 decode_block (const char *data, void *out, int len)
 {
-	memcpy(out, data, len);
+	memcpy (out, data, len);
 	return data+len;
 }
 
 static const char *
 decode_header (const char *data, GnomeMetaFileHeader *mh)
 {
-	data = decode_block(data, mh->signature, sizeof(mh->signature));
-	data = decode_int_header(data, &mh->size);
+	data = decode_block (data, mh->signature, sizeof (mh->signature));
+	data = decode_int_header (data, &mh->size);
 	return data;
 }
 
 static const char *
 decode_page_header (const char *data, GnomeMetaPageHeader *mh)
 {
-	data = decode_block(data, mh->signature, sizeof(mh->signature));
-	data = decode_int_header(data, &mh->size);
+	data = decode_block (data, mh->signature, sizeof (mh->signature));
+	data = decode_int_header (data, &mh->size);
 	return data;
 }
 
 static const char *
-locate_page_header(const char *meta_stream, int page)
+locate_page_header (const char *meta_stream, int page)
 {
 	const char *data = meta_stream + FILEHEADER_SIZE, *next;
 	GnomeMetaPageHeader page_header;
 
-	next = decode_page_header(data, &page_header);
+	next = decode_page_header (data, &page_header);
 	while (page>0 && page_header.size != -1) {
 		data = next + page_header.size;
-		next = decode_page_header(data, &page_header);
+		next = decode_page_header (data, &page_header);
 		page--;
 	}
 	if (page_header.size == -1)
@@ -893,10 +991,10 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 	const char *end = data + size;
 	double matrix [6];
 	double x1, y1, x2, y2, x3, y3;
-	double x, y;
+	double x, y, o;
 	double r, g, b;
 	GnomeMetaPageHeader page_header;
-	
+
 	/*
 	 * Output if all pages were requested, or the first page was
 	 *
@@ -905,24 +1003,28 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 	 */
 	while (data < end){
 		gint32 opcode, i;
-		
+#ifdef META_DEBUG
+		int start_offset;
+		start_offset = size - (end - data);
+#endif
+
 		data = decode_int (data, &opcode);
 #ifdef META_DEBUG
 		if (opcode >= 0 &&
-		    opcode < sizeof (meta_type_names)/sizeof(char *))
+		    opcode < sizeof (meta_type_names)/sizeof (char *))
 			g_warning ("Meta op: '%s' starts at offset 0x%x",
 				   meta_type_names [opcode],
-				   size - (end - data) - 4);
+				   start_offset);
 		else
 			g_warning ("Meta op: OOB %d starts at offset 0x%x",
-				   opcode, size - (end - data) - 4);
+				   opcode, start_offset);
 #endif
 
 		switch ((GnomeMetaType) opcode){
 		case GNOME_META_NEWPATH:
 			gnome_print_newpath (dest);
 			break;
-			
+
 		case GNOME_META_MOVETO:
 			data = decode_double (data, &x);
 			data = decode_double (data, &y);
@@ -934,7 +1036,7 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			data = decode_double (data, &y);
 			gnome_print_lineto (dest, x, y);
 			break;
-			
+
 		case GNOME_META_CURVETO:
 			data = decode_double (data, &x1);
 			data = decode_double (data, &y1);
@@ -945,7 +1047,7 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 
 			gnome_print_curveto (dest, x1, y1, x2, y2, x3, y3);
 			break;
-			
+
 		case GNOME_META_CLOSEPATH:
 			gnome_print_closepath (dest);
 			break;
@@ -956,30 +1058,30 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			data = decode_double (data, &b);
 			gnome_print_setrgbcolor (dest, r, g, b);
 			break;
-			
+
 		case GNOME_META_FILL:
 			gnome_print_fill (dest);
 			break;
-			
+
 		case GNOME_META_EOFILL:
 			gnome_print_eofill (dest);
 			break;
-			
+
 		case GNOME_META_SETLINEWIDTH:
 			data = decode_double (data, &x);
 			gnome_print_setlinewidth (dest, x);
 			break;
-			
+
 		case GNOME_META_SETMITERLIMIT:
 			data = decode_double (data, &x);
 			gnome_print_setmiterlimit (dest, x);
 			break;
-			
+
 		case GNOME_META_SETLINEJOIN:
 			data = decode_int (data, &i);
 			gnome_print_setlinejoin (dest, i);
 			break;
-			
+
 		case GNOME_META_SETLINECAP:
 			data = decode_int (data, &i);
 			gnome_print_setlinecap (dest, i);
@@ -998,49 +1100,40 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			g_free (values);
 			break;
 		}
-		
+
 		case GNOME_META_STROKEPATH:
 			gnome_print_strokepath (dest);
 			break;
-			
+
 		case GNOME_META_STROKE:
 			gnome_print_stroke (dest);
 			break;
-			
+
 		case GNOME_META_SETFONT: {
 			GnomeFont *font;
 			char *name;
-			
-			data = decode_double (data, &x);
-			data = decode_int (data, &i);
 
-			name = g_malloc (i + 1);
-			memcpy (name, data, i);
-			name [i] = 0;
+			data = decode_double (data, &x);
+			data = decode_string (data, &name);
 			font = gnome_font_new (name, x);
 			g_free (name);
-			data += i;
-			
+
 			if (font) {
 				gnome_print_setfont (dest, font);
 				gtk_object_unref (GTK_OBJECT (font));
 			}
 			break;
 		}
-			
+
 		case GNOME_META_SHOW: {
-			char *buffer;
-			
-			data = decode_int (data, &i);
-			buffer = g_malloc (i + 1);
-			memcpy (buffer, data, i);
-			buffer [i] = 0;
-			gnome_print_show (dest, buffer);
-			g_free (buffer);
-			data += i;
+			char *s;
+
+			data = decode_string (data, &s);
+			gnome_print_show (dest, s);
+			g_free (s);
 			break;
 		}
-		
+
 		case GNOME_META_CONCAT:
 		case GNOME_META_SETMATRIX:
 			data = decode_double (data, &matrix [0]);
@@ -1054,36 +1147,36 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			else
 				gnome_print_setmatrix (dest, matrix);
 			break;
-			
+
 		case GNOME_META_GSAVE:
 			gnome_print_gsave (dest);
 			break;
-			
+
 		case GNOME_META_GRESTORE:
 			gnome_print_grestore (dest);
 			break;
-			
+
 		case GNOME_META_CLIP:
 			gnome_print_clip (dest);
 			break;
-			
+
 		case GNOME_META_EOCLIP:
 			gnome_print_eoclip (dest);
 			break;
-			
+
 		case GNOME_META_GRAYIMAGE:
 			data = decode_image (data, dest, 1);
 			break;
-			
+
 		case GNOME_META_RGBIMAGE:
 			data = decode_image (data, dest, 3);
 			break;
-			
+
 		case GNOME_META_SHOWPAGE:
 			gnome_print_showpage (dest);
 
 			/* this is unused here ... */
-			data = decode_page_header(data, &page_header);
+			data = decode_page_header (data, &page_header);
 
 			if (pages!=-1) {
 				pages--;
@@ -1093,25 +1186,30 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			break;
 
 		case GNOME_META_BEGINPAGE: {
-			char *buffer;
+			char *name;
 
-			data = decode_int (data, &i);
-			buffer = g_malloc (i + 1);
-			memcpy (buffer, data, i);
-			buffer [i] = 0;
-			gnome_print_beginpage (dest, buffer);
-			g_free (buffer);
-			data += i;
+			data = decode_string (data, &name);
+			gnome_print_beginpage (dest, name);
+			g_free (name);
 			break;
 		}
-			
+
 		case GNOME_META_TEXTLINE:
 			g_warning ("FIXME: Text line missing");
 			break;
-			
+
 		case GNOME_META_CLOSE:
 			return TRUE;
 
+		case GNOME_META_SETOPACITY:
+			data = decode_double (data, &o);
+			gnome_print_setopacity (dest, o);
+			break;
+
+		case GNOME_META_RGBAIMAGE:
+			data = decode_image (data, dest, 4);
+			break;
+
 		default:
 			g_warning ("Serious print meta data corruption %d",
 				   opcode);
@@ -1141,7 +1239,7 @@ gnome_print_meta_render (GnomePrintContext *destination, const void *meta_stream
 	g_return_val_if_fail (meta_stream != NULL, FALSE);
 
 	data = meta_stream;
-	data = decode_header(data, &header);
+	data = decode_header (data, &header);
 	if (strncmp (header.signature, GNOME_METAFILE_SIGNATURE, GNOME_METAFILE_SIGNATURE_SIZE) != 0)
 		return FALSE;
 
@@ -1198,7 +1296,7 @@ gnome_print_meta_render_page (GnomePrintContext *destination, const void *meta_s
 	g_return_val_if_fail (meta_stream != NULL, FALSE);
 
 	data = meta_stream;
-	data = decode_header(data, &header);
+	data = decode_header (data, &header);
 	if (strncmp (header.signature, GNOME_METAFILE_SIGNATURE, GNOME_METAFILE_SIGNATURE_SIZE) != 0)
 		return FALSE;
 
@@ -1207,7 +1305,7 @@ gnome_print_meta_render_page (GnomePrintContext *destination, const void *meta_s
 		return FALSE;
 	}
 
-	page_data = locate_page_header(meta_stream, page);
+	page_data = locate_page_header (meta_stream, page);
 	if (page_data == NULL) {
 		g_warning ("Trying to print a non-existant page");
 		return FALSE;
@@ -1246,4 +1344,3 @@ gnome_print_meta_pages (const GnomePrintMeta *meta)
 
 	return meta->pages;
 }
-
