@@ -8,8 +8,6 @@
 #include <libgnomeprint/gnome-print-ps.h>
 #include <libgnomeprint/gnome-font.h>
 
-void gnome_print_ps_reencode_font (GnomePrintContext *pc, const char* fontname);
-
 static void gnome_print_ps_class_init (GnomePrintPsClass *klass);
 
 static void
@@ -18,6 +16,9 @@ gnome_print_ps_init (GnomePrintPs *ps);
 static void
 gnome_print_ps_finalize (GtkObject *object);
 
+static void
+gnome_print_ps_reencode_font (GnomePrintContext *pc, const char *fontname);
+
 static GnomePrintContextClass *parent_class = NULL;
 
 GtkType
@@ -198,6 +199,11 @@ gnome_print_ps_stroke (GnomePrintContext *pc)
 static int
 gnome_print_ps_setopacity (GnomePrintContext *pc, double opacity)
 {
+  static gboolean warned = FALSE;
+  if (!warned) {
+    g_warning ("Unimplmented setopacity");
+    warned = TRUE;
+  }
   return 0;
 }
 
@@ -362,8 +368,8 @@ gnome_print_ps_setfont (GnomePrintContext *pc, GnomeFont *font)
 
 }
 
-void
-gnome_print_ps_reencode_font (GnomePrintContext *pc, const char * fontname)
+static void
+gnome_print_ps_reencode_font (GnomePrintContext *pc, const char *fontname)
 {
   gnome_print_context_fprintf (pc,"/%s findfont\ndup length dict begin\n"
 			       "{1 index /FID ne {def} {pop pop} ifelse}"
@@ -710,6 +716,7 @@ gnome_print_ps_class_init (GnomePrintPsClass *class)
   pc_class->textline = gnome_print_ps_textline;
   pc_class->showpage = gnome_print_ps_showpage;
   pc_class->beginpage = gnome_print_ps_beginpage;
+  pc_class->setopacity = gnome_print_ps_setopacity;
 
   pc_class->close = gnome_print_ps_close;
 }
