@@ -104,7 +104,7 @@ gnome_printer_stock_profile (void)
 	GnomePrinterProfile *pp;
 
 	pp = g_new0 (GnomePrinterProfile, 1);
-	pp->name = g_strdup ("Generic Postscript 2");
+	pp->name = g_strdup ("Generic Postscript");
 	pp->driver = g_strdup ("gnome-print-ps");
 	pp->mime_type = g_strdup ("application/postscript");
 	pp->output = g_strdup ("output.ps");
@@ -289,7 +289,7 @@ gnome_printer_profile_get_printer (GnomePrinterProfile *pp,
 {
 	GnomePrinter *printer = NULL;
 	char *output = NULL;
-	
+
 	g_return_val_if_fail (pp != NULL, printer);
 
 	if (optional_file != NULL && optional_command != NULL){
@@ -297,24 +297,24 @@ gnome_printer_profile_get_printer (GnomePrinterProfile *pp,
 		return NULL;
 	}
 
-	if (strncmp (pp->output, "file", 4) == 0){
-		output = g_strdup (pp->output + 5);
-	} else if (strncmp (pp->output, "command", 7) == 0){
-		char *cmd = pp->output + 8;
-		
-		output = g_malloc (strlen (cmd) + 2);
-		output [0] = '|';
-		strcpy (&output [1], cmd);
-	}
-
 	if (optional_file){
-		if (output)
-			g_free (output);
+		/*
+		 * FIXME: do something about funny filenames?  Like one
+		 * starting with a '|'.
+		 */
 		output = g_strdup (optional_file);
 	} else if (optional_command) {
 		output = g_malloc (strlen (optional_command) + 2);
-		output [0] = '|';
-		strcpy (&output [1], optional_command);
+		output[0] = '|';
+		strcpy (&output[1], optional_command);
+	} else if (strncmp (pp->output, "file", 4) == 0){
+		output = g_strdup (pp->output + 5);
+	} else if (strncmp (pp->output, "command", 7) == 0){
+		const char *cmd = pp->output + 8;
+
+		output = g_malloc (strlen (cmd) + 2);
+		output[0] = '|';
+		strcpy (&output[1], cmd);
 	}
 
 	if (!output)
