@@ -2948,6 +2948,9 @@ gt1_glyph_outline_lookup (Gt1LoadedFont *font, int c)
 
 	go->ch = c;
 	go->bpath = gt1_get_glyph_outline (font, c, &go->wx);
+	if (!go->bpath)
+		return NULL;
+
 	art_drect_bpath (&go->bbox, go->bpath);
 
 	g_hash_table_insert (font->glyph_outlines, go, go);
