@@ -102,7 +102,7 @@ print_test_page (GnomePrinter *printer)
   int x, y;
   int i;
 
-  pc = gnome_print_context_new (printer, "US-Letter");
+  pc = gnome_print_context_new_with_paper_size (printer, "US-Letter");
 
   line = get_line ("Times", "\302\241hello, world firefly TOYOTA!", 36);
 
